#<profile4xi profile = "03" />
#<program4xi program = "duphandle4xi.py">
#<abstract4xi abstract = "providing class DupHandle4Xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'DupHandle4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                    # system ones
    import os
    import sys
    import signal
except:
    pass
try:                    # threading ones       
    import threading                
    import _thread   
except:
    pass
try:                    # winapi
    import _winapi
    from _winapi import CloseHandle
    from _winapi import DuplicateHandle
    from _winapi import DUPLICATE_SAME_ACCESS
    from _winapi import CreateProcess
    from _winapi import GetCurrentProcess
    from _winapi import GetExitCodeProcess
    from _winapi import INFINITE
    from _winapi import TerminateProcess
    from _winapi import WaitForSingleObject
    from _winapi import WAIT_OBJECT_0
    from _winapi import WAIT_TIMEOUT    
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.reduction import DupHandle
except:
    print("duphandle4xi.py: *** failed in from multiprocessing.reduction import DupHandle ***")
               
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

from examine4xi import examine4xi

from timeout4xi import omega2timeout4xi

from resourcesharer4xi import ResourceSharer4Xi 

from print4xi import print4xi
from time4xi import time4xi   


#"
#/>

# =================================================================================================
#<class4xi class = "DupHandle4Xi" base = "object" child = "None" synopsis = "duplicates handle for process with given pid" usage = "
#   send_handle4xi.py:
#       duplicatehandler2send = DupHandle4Xi(handle, _winapi.DUPLICATE_SAME_ACCESS, destination_pid)
#>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class DupHandle4Xi(object):  
    '''
    DupHandle4Xi class duplicates handle for process with given pid
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('detach',
                 'sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    resource_sharer2xi = ResourceSharer4Xi()
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupHandle4Xi" usage = "initializes DupHandle4Xi with (self, handle, access, pid=None)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, handle, access, pid=None):
        '''
        initializes DupHandle4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupHandle4Xi"
        method2print = "__init__"
        message2print = "{(Step 1/10a) started" + "; init-of-DupHandle4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                                   
        message2print = "{(Step 1/10b) passed" + "; handle-passed-to-init-of-DupHandle4Xi: " + str(handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        message2print = "{(Step 1/10c) passed" + "; access-passed-to-init-of-DupHandle4Xi: " + str(access) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #
         
        message2print = "{(Step 1/10d) passed" + "; pid-passed-to-init-of-DupHandle4Xi: " + str(pid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 

            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupHandle-init-performing-by-duplicate4xi: " + str(DupHandle.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            DupHandle.__init__(self, handle, access, pid)
            
            message2print = "{(StepOriginal) performed" + "; Original-DupFd-init-performed-by-duplicate4xi: " + str(DupHandle.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        else:        
                                        
            if pid is None:
                pid = os.getpid()
                message2print = "{(Step 1/10e) set" + "; pid-set-by-init-of-DupHandle4Xi to: " + str(pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
            # -----------------------------------------------------------------------------------------
            # (Step 2/10)            
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 2/10a) performing" + "; winapi-OpenProcess-performing-by-init-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
            process2xi = None
                            
            try:
                message2print = "{(Step 2/10b) passing" + "; winapi-PROCESS_DUP_HANDLE-passing-to-winapi-OpenProcess-by-init-of-DupHandle4Xi: " + str(_winapi.PROCESS_DUP_HANDLE) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #
                
                message2print = "{(Step 2/10c) passing" + "; False-passing-to-winapi-OpenProcess-by-init-of-DupHandle4Xi: " + str(False) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                message2print = "{(Step 2/10d) passing" + "; pid-passing-to-winapi-OpenProcess-by-init-of-DupHandle4Xi: " + str(pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                process2xi = _winapi.OpenProcess(_winapi.PROCESS_DUP_HANDLE, False, pid)    
                
                message2print = "{(Step 2/10e) performed" + "; winapi-OpenProcess-performed-by-init-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 2/10f) got" + "; process2xi-got-by-init-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                
            except:
                message2print = "{(Step 2/10z1) failed" + "; winapi-OpenProcess-failed-by-init-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            # -----------------------------------------------------------------------------------------
            # (Step 3/10)            
            # -----------------------------------------------------------------------------------------    
            handle2currentprocess = None
            if process2xi is None:
                message2print = "{(Step 3/10a) failed" + "; process2xi-failed-by-init-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                     
            else:
                
                try:
                    handle2currentprocess = _winapi.GetCurrentProcess()
                    
                    message2print = "{(Step 3/10a) got" + "; handle2currentprocess-got-by-init-of-DupHandle4Xi: " + str(handle2currentprocess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 3/10z1) failed" + "; handle2currentprocess-failed-by-init-of-DupHandle4Xi: " + str(handle2currentprocess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                     
                if handle2currentprocess is None:
                    self._handle = None
                    message2print = "{(Step 3/10z2) failed" + "; handle-failed-by-init-of-DupHandle4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                else:
                    try:
                        message2print = "{(Step 3/10b1) passing" + "; handle2currentprocess-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(handle2currentprocess) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        message2print = "{(Step 3/10b2) passing" + "; handle-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(handle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #                              
                        message2print = "{(Step 3/10b3) passing" + "; process2xi-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #  
                        message2print = "{(Step 3/10b4) passing" + "; access-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(access) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #  
                        message2print = "{(Step 3/10b5) passing" + "; False-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(False) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #  
                        message2print = "{(Step 3/10b6) passing" + "; 0-passing-to-winapi-DuplicateHandle-by-init-of-DupHandle4Xi: " + str(0) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #                                                                          
                        
                                                
                        self._handle = _winapi.DuplicateHandle(handle2currentprocess,handle, process2xi, access, False, 0)          
                        message2print = "{(Step 3/10b7) set" + "; handle-of-duphandle2xi-set-by-init-of-DupHandle4Xi to: " + str(self._handle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                    except:
                        self._handle = None
                        
                        message2print = "{(Step 3/10bz) failed" + "; handle-of-duphandle2xi-failed-by-init-of-DupHandle4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                    finally:
                        message2print = "{(Step 3/10b8) passing" + "; process2xi-passing-to-winapi-CloseHandle-by-init-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        _winapi.CloseHandle(process2xi)
                        
                    self._access = access
                    message2print = "{(Step 3/10b9) set" + "; access-of-duphandle2xi-set-by-init-of-DupHandle4Xi to: " + str(self._access) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    self._pid = pid        
                    message2print = "{(Step 3/10b10) set" + "; pid-of-duphandle2xi-set-by-init-of-DupHandle4Xi to: " + str(self._pid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #     
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 10/10) 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 10/10a) activated" + "; dubhandle2xi-of-DupHandle4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #                 
            
            message2print = "{(Step 10/10b) inherited" + "; object-inherited-by-dubhandle2xi-of-DupHandle4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #    
             
        
            message2print = "{(Step 10/10c) returning" + "; init-of-DupHandle4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #                        
                
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "detach-of-DupHandle4Xi" usage = "retrieves handle from process which currently owns it">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def detach(self):
        '''
        retrieves handle from process which currently owns it
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupHandle4Xi"
        method2print = "detach"
        message2print = "{(Step 1/10a) started" + "; detach-of-DupHandle4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
    
            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupHandle-detach-performing-by-duplicate4xi: " + str(DupHandle.detach) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            return DupHandle.detach(self)

        else:          
                                   
            # -----------------------------------------------------------------------------------------
            # (Step 2/10)            
            # -----------------------------------------------------------------------------------------                                     
            if self._pid == os.getpid():
                message2print = "{(Step 2/10a) using" + "; pid-of-duphandle2xi-using-by-detach-of-DupHandle4Xi: " + str(self._pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            
            
                message2print = "{(Step 2/10b) returning" + "; handle-of-duphandle2xi-returning-by-detach-of-DupHandle4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            
                return self._handle
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/10)            
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 3/10a) performing" + "; winapi-OpenProcess-performing-by-detach-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
            process2xi = None
                            
            try:
                message2print = "{(Step 2/10b) passing" + "; winapi-PROCESS_DUP_HANDLE-passing-to-_winapi.OpenProcess-by-detach-of-DupHandle4Xi: " + str(_winapi.PROCESS_DUP_HANDLE) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #
                
                message2print = "{(Step 2/10c) passing" + "; False-passing-to-_winapi.OpenProcess-by-detach-of-DupHandle4Xi: " + str(False) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                     
                message2print = "{(Step 2/10d) passing" + "; pid-of-duphandle2xi-passing-to-_winapi.OpenProcess-by-detach-of-DupHandle4Xi: " + str(self._pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                    
                process2xi = _winapi.OpenProcess(_winapi.PROCESS_DUP_HANDLE, False, self._pid)    
                
                message2print = "{(Step 2/10e) performed" + "; winapi-OpenProcess-performed-by-detach-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 2/10f) got" + "; process2xi-got-by-detach-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                    
            except:
                message2print = "{(Step 2/10z1) failed" + "; winapi-OpenProcess-failed-by-detach-of-DupHandle4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
    
            # -----------------------------------------------------------------------------------------
            # (Step 4/10)            
            # -----------------------------------------------------------------------------------------    
            if process2xi is None:
                message2print = "{(Step 4/10a) failed" + "; process2xi-failed-by-detach-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                     
            else:
                duplicatehandle2currentprocess = None
                try:
                    currentprocess2xi = _winapi.GetCurrentProcess()
                    message2print = "{(Step 4/10b) got" + "; currentprocess2xi-got-by-detach-of-DupHandle4Xi: " + str(currentprocess2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                        
                    message2print = "{(Step 4/10c1) passing" + "; process2xi-passing-to-winapi-DuplicateHandle-by-detach-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 4/10c2) passing" + "; handle-of-duphandle2xi-passing-to-winapi-DuplicateHandle-by-detach-of-DupHandle4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 4/10c3) passing" + "; currentprocess2xi-passing-to-winapi-DuplicateHandle-by-detach-of-DupHandle4Xi: " + str(currentprocess2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 4/10c4) passing" + "; access-of-duphandle2xi-passing-to-winapi-DuplicateHandle-by-detach-of-DupHandle4Xi: " + str(self._access) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 4/10c5) passing" + "; False-passing-to-winapi-DuplicateHandle-by-detach-of-DupHandle4Xi: " + str(False) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 4/10c6) passing" + "; winapi-DUPLICATE_CLOSE_SOURCE-passing-to-winapi-OpenProcess-by-detach-of-DupHandle4Xi: " + str(_winapi.DUPLICATE_CLOSE_SOURCE) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                              
                    duplicatehandle2currentprocess = _winapi.DuplicateHandle(process2xi, self._handle, currentprocess2xi, self._access, False, _winapi.DUPLICATE_CLOSE_SOURCE)
                    
                    message2print = "{(Step 4/10c7) got" + "; duplicatehandle2currentprocess-got-by-detach-of-DupHandle4Xi: " + str(duplicatehandle2currentprocess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    message2print = "{(Step 4/10c8) returning" + "; duplicatehandle2currentprocess-returning-by-detach-of-DupHandle4Xi: " + str(duplicatehandle2currentprocess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    return duplicatehandle2currentprocess
                     
                except:
                    message2print = "{(Step 4/10z1) failed" + "; duplicatehandle2currentprocess-failed-by-detach-of-DupHandle4Xi: " + str(duplicatehandle2currentprocess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                finally:
                    message2print = "{(Step 3/10b8) passing" + "; process2xi-passing-to-winapi-CloseHandle-by-detach-of-DupHandle4Xi: " + str(process2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    _winapi.CloseHandle(process2xi)
                        
                    message2print = "{(Step 3/10b9) performed" + "; winapi-CloseHandle-performed-by-detach-of-DupHandle4Xi: " + str(_winapi.CloseHandle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 

    #</methodize4xi>    
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-DupHandle4Xi" usage = "senses DupHandle4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses DupHandle4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = 'DupHandle4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-DupHandle4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; DupHandle4Xi-sensing-by-sense-of-DupHandle4Xi: " + str(DupHandle4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
            
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-DupHandle4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; DupHandle4Xi-sensed-by-sense-of-DupHandle4Xi: " + str(DupHandle4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-DupHandle4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
          
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-DupHandle4Xi" usage = "reasons DupHandle4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons DupHandle4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = 'DupHandle4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-DupHandle4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; DupHandle4Xi-reasoning-by-reason-of-DupHandle4Xi: " + str(DupHandle4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-DupHandle4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
            
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
            
            
              
            
        message2print = "{(Step 4/5) reasoning" + "; DupHandle4Xi-reasoning-by-reason-of-DupHandle4Xi: " + str(DupHandle4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-DupHandle4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
             
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-DupHandle4Xi" usage = "aces DupHandle4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces DupHandle4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = 'DupHandle4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-DupHandle4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; DupHandle4Xi-acing-by-ace-of-DupHandle4Xi: " + str(DupHandle4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
            
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-DupHandle4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; DupHandle4Xi-acing-by-ace-of-DupHandle4Xi: " + str(DupHandle4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-DupHandle4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>

#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of duphandle4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

